/* Program  specabstorel.cc      */
/* By Harald Stauss              */
/* Last change 29. August 1997   */

#include <stdlib.h>
#include <ctype.h>
#include "tsa.h"

main(int argc, char *argv[]) {
  short    i, chn, freq, rc=0;
  unsigned long     n;
  char     *fin, *fout, *ftin, *ftout, *pftin, *pftout;
  REGIST   in, out;
  float    sr, *variance, *time, **data, *min, *max;

  /* Hello world */
  printf("specabstorel Vein.1.0 by H. Stauss, 1997 \n");

  /* A r r a y s   a l l o c i e r e n */
  fin =new char[256];
  fout=new char[256];
  ftin =new char[256];
  ftout=new char[256];

  if ((argc!=4) && (argc!=5)) {
    cout << "number of parameters wrong! \n";
    cout << "calculates relative spectrum from absolute spectrum \n";
    cout << "specabstorel fin fout freq [sr] \n";
    cout << "fin    : name of input file (*.asc). \n";
    cout << "fout   : name of output file (*.tsa). \n";
    cout << "freq   : 1: frequency axis in first column of output file \n";
    cout << "         0: no frequency axis \n";
    cout << "[sr]   : sampling rate if input file ASCII \n";
    rc=-1;
    goto ende;
  }

  /* Parameter einlesen */
  strcpy(fin ,argv[1]);
  strcpy(ftin, fin);
  strcpy(fout,argv[2]); 
  strcpy(ftout, fout);
  freq=atoi(argv[3]);
  if (argc==5) sr=atof(argv[4]);

  cout << "input file : " << fin << "\n";
  cout << "output file: " << fout << "\n";
  if (freq) cout << "frequency axis will be in first column \n";
  else      cout << "no frequency axis \n";

  /* Datei einlesen */
  pftin=strrchr(ftin,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);
  if ((!strcmp(pftin,".ASC")) && (argc!=5)) {
    cout << "sampling rate must be given for ASCII files \n";
    rc=-1;
    goto ende;
  }
  if (strcmp(pftin,".TSA") && strcmp(pftin,".ASC")) {
    cout << "file type for input not recognized! \n";
    rc=-1;
    goto ende;
  }
  if (!strcmp(pftin,".TSA")) in.ReadTSA(fin);
  if (!strcmp(pftin,".ASC")) in.ReadASCII(fin,sr);

  cout << "number of channels: " << in.n << "\n";
  cout << "sampling rate     : " << in.ch[0].sr << "\n";

  /* Allocate memory for output data */
  data=new (float *)[in.n];
  time=new float[in.ch[0].n];
  variance=new float[in.n];
  min=new float[in.n];
  max=new float[in.n];
  for (i=0; i<in.n; i++)
    data[i]=new float[in.ch[i].n];

  /* calculate variances */
  for (i=0; i<in.n; i++) {
    variance[i]=0;
    for (n=0; n<in.ch[i].n; n++) variance[i]+=in.ch[i].data[n];
    cout << "number of values in channel " << i << " " << in.ch[i].n <<
      " variance: " << variance[i] << "\n";
  }

  /* make time (or frequency) */
  if (freq) 
    for (n=0; n<in.ch[0].n; n++) 
      time[n]=n*(in.ch[0].sr/2.0)/(float)(in.ch[0].n-1.0);
  
  /* calculate relative spectra find min and max */
  for (i=0; i<in.n; i++) {
    min[i]=1e30; max[i]=-1e30;
    for (n=0; n<in.ch[i].n; n++) {
      if (min[i]>in.ch[i].data[n]) min[i]=in.ch[i].data[n];
      if (max[i]<in.ch[i].data[n]) max[i]=in.ch[i].data[n];
      data[i][n]=in.ch[i].data[n]/variance[i];
    }
  }

  /* add new channels */
  if (freq) {
    out.AddChannel(in.ch[0].n, in.ch[0].sr, 0, time[in.ch[0].n-1],
		   sizeof(float), "frequency", "Hz");
    for (n=0; n<in.ch[0].n; n++) out.ch[0].data[n]=time[n];
  }
  for (i=0; i<in.n; i++) {
    if (freq) chn=i+1; else chn=i;
    out.AddChannel(in.ch[i].n, in.ch[i].sr, min[i], max[i],
		   sizeof(float), "name", "unit");
    for (n=0; n<in.ch[i].n; n++) out.ch[chn].data[n]=data[i][n];
  }

  /* write output file */
  pftout=strrchr(ftout,'.');
  for (i=0; pftout[i]; i++) pftout[i]=toupper(pftout[i]);
  if (strcmp(pftout,".TSA") && strcmp(pftout,".ASC")) {
    cout << "file type for output not recognized! \n";
    rc=-1;
    goto memfree;
  }
  if (!strcmp(pftout,".TSA")) out.WriteTSA(fout);
  if (!strcmp(pftout,".ASC")) out.WriteASCII(fout);

  /* free memory */
memfree:
  for (i=0; i<in.n; i++) delete[]data[i];
  delete[]data;
  delete[]variance;
  delete[]min;
  delete[]max;
  delete[]time;

ende:
  delete[]fin; delete[]fout;
  delete[]ftin; delete[]ftout;
  return(rc);
} /* Ende von main */

