/* Program SPLINE.CC */
/* By Harald Stauss */
/* Last change 01.04.03 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "tsa.h"
#include "../nrc/nrutil.h" 
#include "../nrc/nrutil.cc"
#include "../nrc/four1.cc"
#include "../nrc/moment.cc"
#include "../nrc/spline.cc"
#include "../nrc/splint.cc"

main(int argc, char *argv[])
{
  short          i, ch;
  unsigned long  nneu, fn, n, eospec;
  float          ave, adev, sdev, var, skew, curt;
  float          *data, *time, *y2;
  float          srin, fifreq, srout;
  float          t,dt,wert;
  char           *fin,*fout,*ftin,*ftout, *pftin, *pftout;
  REGIST         r,f;


  // H e l l o   w o r l d;
  cout << "SPLINE Ver.1.1 by H. Stauss, 2003 \n";

  // A r r a y s   a l l o c i e r e n ;
  fin=(char *)malloc(255);
  fout=(char *)malloc(255);
  ftin=(char *)malloc(255);
  ftout=(char *)malloc(255);

  data=(float *) malloc(sizeof(float));
  time=(float *) malloc(sizeof(float));
  y2=(float *) malloc(sizeof(float));

  /* P a r a m e t e r   e i n l e s e n */
  if (argc != 7 && argc!=8)
    {
      cout << "Number of parameters wrong! \n";
      cout << "spline f_in f_out ch n f srout [srin] \n";
      cout << "removes a linear trend, \n";
      cout << "zero pads to n values if necessary, \n";
      cout << "calculates the FFT and cuts the spectrum at frequency f, \n";
      cout << "performs a reverse FFT and resamples at srout by spline. \n";
      cout << "f_in  : name of input file (*.TSA, *.ASC). \n";
      cout << "f_out : name of output file (*.TSA, *.ASC). \n";
      cout << "ch    : channel number 1-i, 0 = all channels. \n";
      cout << "n     : number of values for FFT (0: no filter). \n";
      cout << "f     : frequency for low-pass filter (0: no filter). \n";
      cout << "srout : output sampling rate. \n";
      cout << "srin  : input sampling rate (optional ASCII files) \n";
      goto ende;
    }
  strcpy(fin,argv[1]);
  strcpy(fout,argv[2]);
  strcpy(ftin,fin);
  strcpy(ftout,fout);
  pftin=strrchr(ftin,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);
  pftout=strrchr(ftout,'.');
  for (i=0; pftout[i]; i++) pftout[i]=toupper(pftout[i]);
  sscanf(argv[3],"%d",&ch);
  sscanf(argv[4],"%ul",&fn);
  sscanf(argv[5],"%f",&fifreq);
  sscanf(argv[6],"%f",&srout); 
  if (argc==8) sscanf(argv[7],"%f",&srin);
  if ((strcmp(ftin,".ASC")==0) && (argc!=8)) {
    cout << "Need sampling rate for ASCII Files!\n";
    goto ende;
  }

  printf("Input file              : %s \n",fin);
  printf("Output file             : %s \n",fout);
  cout << "Number of values for FFT: " << fn << "\n";
  cout << "Frquency for filter     : " << fifreq << "\n";

  /* E i n g a b e d a t e i   o e f f n e n */
  if (strcmp(pftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(pftin,".ASC")==0) r.ReadASCII(fin,srin);
  if ((strcmp(pftin,".TSA")!=0) && (strcmp(pftin,".ASC")!=0)) {
    cout << "File type for input not recognized! \n";
    goto ende;
  }
  printf("Number of channels      : %d  \n",r.n);

  /* M A I N L O O P  */
  for (i=0; i < r.n; i++) {
    if (i==(ch-1) || ch==0) {
      srin=r.ch[i].sr;
      cout << "Channel: " << i << " Sampling rate in        : " << srin << "\n";
      cout << "Channel: " << i << " Sampling rate out       : " << srout << "\n";
      if (fn==0 || fifreq==0) {
	fn=r.ch[i].n;
	data=(float *)realloc(data,2*fn*sizeof(float));
	time=(float *)realloc(time,fn*sizeof(float));
	y2=(float *)realloc(y2,fn*sizeof(float));
	for (n=0; n<fn; n++) {
	  time[n]=n/srin;
	  data[n]=r.ch[i].data[n];
	}
	spline(time-1,data-1, fn, 1e30, 1e30, y2-1);
	// Neue Registrierung fuellen;
	f.AddChannel((unsigned long)((float)r.ch[i].n*srout/srin),
		     srout, 1e30, 1e-30, 
		     r.ch[i].height, r.ch[i].color,
		     r.ch[i].name, r.ch[i].unit);
	t=0; dt=1/srout;
	for(n=0; n<(unsigned long)((float)r.ch[i].n*srout/srin); n++) {
	  splint(time-1, data-1, y2-1, fn, t, &wert);
	  f.ch[i].data[n]=wert;
	  t+=dt;
	  if (wert>f.ch[i].max) f.ch[i].max=wert;
	  if (wert<f.ch[i].min) f.ch[i].min=wert;
	}
      }
      else {
	cout << "n in channel " << i+1 << ": " << r.ch[i].n <<
	  " FFT based on " << fn << " values ";
	if (fn<=r.ch[i].n) cout << "\n"; else cout << "=> zero padded! \n";
	// allocate memory;
	data=(float *)realloc(data,2*fn*sizeof(float));
	time=(float *)realloc(time,fn*sizeof(float));
	y2=(float *)realloc(y2,fn*sizeof(float));
	// Mittelwert berechnen;
	n=fn;
	if (r.ch[i].n<fn) n=r.ch[i].n;
	moment(r.ch[i].data-1,n,&ave,&adev,&sdev,&var,&skew,&curt);
	/* Daten fuer FFT kopieren */
	for (n=0; n<fn; n++) {
	  if (n<r.ch[i].n) { 
	    data[n*2]=r.ch[i].data[n]-ave;
	    data[n*2+1]=0;
	  }
	  else {
	    data[n*2]=0;
	    data[n*2+1]=0;
	  }
	}
	four1(data-1,fn,1);
	// Spectrum abschneiden;
	data[0]=0; data[1]=0;
	eospec=(unsigned long)(fn*fifreq/srin);
	for (n=eospec; n<=fn-eospec; n++) {
	  data[n*2]=0;
	  data[n*2+1]=0;
	}
	// Ruecktransformieren;
	four1(data-1,fn,-1);
	// spline;
	for (n=0; n<fn; n++) {
	  time[n]=n/srin;
	  data[n]=data[2*n]/fn+ave;
	}
	spline(time-1,data-1, fn, 1e30, 1e30, y2-1);
	// Neue Registrierung fuellen;
	if (fn>r.ch[i].n)
	  nneu=(unsigned long)((float)r.ch[i].n*srout/srin);
	else
	  nneu=(unsigned long)((float)fn*srout/srin);
	f.AddChannel(nneu, srout,
		     r.ch[i].min, r.ch[i].max, 
		     r.ch[i].height, r.ch[i].color,
		     r.ch[i].name, r.ch[i].unit);
	t=0; dt=1/srout;
	for(n=0; n<nneu; n++) {
	  splint(time-1, data-1, y2-1, fn, t, &wert);
	  f.ch[i].data[n]=wert;
	  t+=dt;
	  if (wert>f.ch[i].max) f.ch[i].max=wert;
	  if (wert<f.ch[i].min) f.ch[i].min=wert;
	}
      } /* endelse */
    } /* endif ch */
    else {
      f.AddChannel(r.ch[i].n, r.ch[i].sr,
		   r.ch[i].min, r.ch[i].max,
		   r.ch[i].height, r.ch[i].color,
		   r.ch[i].name, r.ch[i].unit);
      for(n=0; n<r.ch[i].n; n++) f.ch[i].data[n]=r.ch[i].data[n];
    }

  } /* endfor */

  /* D a t e i   s c h r e i b e n */
  if (strcmp(pftout,".TSA")==0) f.WriteTSA(fout);
  if (strcmp(pftout,".ASC")==0) f.WriteASCII(fout);
  if ((strcmp(pftout,".TSA")!=0) && (strcmp(pftout,".ASC")!=0)) {
    printf("File type for output not recognized! \n");
    goto ende;
  }

 ende:
  free(data); free(time); free(y2);
  free(fin);free(ftin);free(fout);free(ftout);
  return(0);
} /* Ende von main */
