/* Program volume.cc    */
/* By Harald Stauss     */
/* Last change 02.03.99 */

#include <ctype.h>
#include "tsa.h"

main(int argc, char *argv[])
{
  int               i;
  float             lfa, lfe, mfa, mfe, hfa, hfe, endf;
  unsigned long     nlfa, nlfe, nmfa, nmfe, nhfa, nhfe, nendf;
  float             sr;
  float             sl,sm,sh,st;
  unsigned long     n;
  char              *fin,*fout,*ftin;
  char              *pfin, *pfout, *pftin;
  char              fname[255];
  REGIST            r;
  ofstream          lfdat, mfdat, hfdat, totdat;

  /* H e l l o   w o r l d */
  cout << "VOLUME Ver.1.2 by H. Stauss, 1999 \n";

  /* A r r a y s   a l l o c i e r e n */
  fin= new char [256];  pfin=fin;
  fout=new char [256]; pfout=fout;
  ftin=new char [256]; pftin=ftin;

  /* P a r a m e t e r   e i n l e s e n */
  if (argc != 10 && argc!=11)
    {
      cout << "Number of parameters wrong! \n";
      cout << "volume f_in f_out lfa lfe mfa mfe hfa hfe endf [sr] \n";
      cout << "calculates area under the curve  \n";
      cout << "f_in  : name of input file  (*.TSA, *.ASC). \n";
      cout << "f_out : name of output file (appended). \n";
      cout << "lfa lfe mfa mfe hfa hfe: frequency boundaries. \n";
      cout << "endf  : frequency up to which total power is calculated. \n";
      cout << "[sr]  : sampling rate optional if *.ASC. \n";
      goto ende;
    }
  strcpy(fin,argv[1]);
  strcpy(fout,argv[2]);
  strcpy(ftin,fin);
  pftin=strrchr(ftin,'.');
  for (i=0; pftin[i]; i++) pftin[i]=toupper(pftin[i]);

  if (strcmp(pftin,".ASC")==0 && argc==9) {
    cout << "Need sampling rate when input file is ASCII! \n";
    goto ende;
  }
  sscanf(argv[3],"%f",&lfa);
  sscanf(argv[4],"%f",&lfe);
  sscanf(argv[5],"%f",&mfa);
  sscanf(argv[6],"%f",&mfe);
  sscanf(argv[7],"%f",&hfa);
  sscanf(argv[8],"%f",&hfe);
  sscanf(argv[9],"%f",&endf);

  cout << "Input file    : " << fin  << "\n";
  cout << "Output file   : " << fout << "\n";

  /* E i n g a b e d a t e i e n   o e f f n e n */
  if (strcmp(pftin,".ASC")==0 && argc==11) sscanf(argv[10],"%f",&sr);
  if (strcmp(pftin,".ASC")==0 && argc==10) {
    cout << "Need sampling rate for ASCII files! \n";
    goto ende;
  }
  if (strcmp(pftin,".TSA")==0) r.ReadTSA(fin);
  if (strcmp(pftin,".ASC")==0) r.ReadASCII(fin,sr);
  if ((strcmp(pftin,".TSA")!=0) && (strcmp(pftin,".ASC")!=0)) {
    cout << "File type for input file not recognized! \n";
    goto ende;
  }

  cout << "Number of channels in file : " << r.n << "\n";
  cout << "Sampling rate              : " << r.ch[0].sr  << "\n";
  cout << "End frequency              : " << endf << "\n";

  /*   A u s g a b e d a t e i e n   */
  strcpy(fname,fout); strcat(fname,"-lf");
  lfdat.open(fname,ios::app);
  if (!lfdat) {
    cout << "open lfdat \n";
    lfdat.open(fname);
  }
  strcpy(fname,fout); strcat(fname,"-mf");
  mfdat.open(fname,ios::app);
  if (!mfdat) {
    cout << "open mfdat \n";
    mfdat.open(fname);
  }
  strcpy(fname,fout); strcat(fname,"-hf");
  hfdat.open(fname,ios::app);
  if (!hfdat) {
    cout << "open hfdat \n";
    hfdat.open(fname);
  }
  strcpy(fname,fout); strcat(fname,"-tot");
  totdat.open(fname,ios::app);
  if (!totdat) {
    cout << "open totdat \n";
    totdat.open(fname);
  }

  /* M A I N L O O P  */
  for (i=0; i<r.n ; i++) {
    nendf=(unsigned long) ((r.ch[i].n-1)*endf/(r.ch[i].sr/2));
    sl=0; sm=0; sh=0; st=0;
    nlfa=(unsigned long) ((r.ch[i].n-1)*lfa/(r.ch[i].sr/2));
    nlfe=(unsigned long) ((r.ch[i].n-1)*lfe/(r.ch[i].sr/2));
    nmfa=(unsigned long) ((r.ch[i].n-1)*mfa/(r.ch[i].sr/2));
    nmfe=(unsigned long) ((r.ch[i].n-1)*mfe/(r.ch[i].sr/2));
    nhfa=(unsigned long) ((r.ch[i].n-1)*hfa/(r.ch[i].sr/2));
    nhfe=(unsigned long) ((r.ch[i].n-1)*hfe/(r.ch[i].sr/2));
    for (n=0; n<nendf; n++) {
      if (n>=nlfa && n<=nlfe) sl+=r.ch[i].data[n];
      if (n>=nmfa && n<=nmfe) sm+=r.ch[i].data[n];
      if (n>=nhfa && n<=nhfe) sh+=r.ch[i].data[n];
      st+=r.ch[i].data[n];
    }
    lfdat  << sl << "\t" << 100*sl/st << "\t";
    mfdat  << sm << "\t" << 100*sm/st << "\t";
    hfdat  << sh << "\t" << 100*sh/st << "\t";
    totdat << st << "\t";
  } /* endfor */
  lfdat  << "\n";
  mfdat  << "\n";
  hfdat  << "\n";
  totdat << "\n";

  lfdat.close();
  mfdat.close();
  hfdat.close();
  totdat.close();

 ende:
  /*  cout << nlfa << " " << nlfe << " " << nmfa << " " << nmfe << " " << nhfa << " " << nhfe << "\n"; */
  free(fin);free(ftin);free(fout);
  return(0);
} /* Ende von main */

